package com.bocloud.cmp.provider.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.RegionModel;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.model.TceSyncModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.model.ZoneModel;
import com.bocloud.cmp.provider.TceProvider;
import com.bocloud.cmp.provider.compute.ImageProvider;
import com.bocloud.cmp.provider.compute.ServerProvider;
import com.bocloud.cmp.provider.compute.SnapshotProvider;
import com.bocloud.cmp.provider.compute.TceImageProvider;
import com.bocloud.cmp.provider.compute.TceServerProvider;
import com.bocloud.cmp.provider.compute.TceSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.network.SecurityGroupProvider;
import com.bocloud.cmp.provider.network.SecurityGroupRuleProvider;
import com.bocloud.cmp.provider.network.SubnetProvider;
import com.bocloud.cmp.provider.network.TceSecuGroupProvider;
import com.bocloud.cmp.provider.network.TceSecuGroupRuleProvider;
import com.bocloud.cmp.provider.network.TceSubnetProvider;
import com.bocloud.cmp.provider.network.TceVpcProvider;
import com.bocloud.cmp.provider.network.VpcProvider;
import com.bocloud.cmp.provider.security.KeypairProvider;
import com.bocloud.cmp.provider.security.TceKeypairProvider;
import com.bocloud.cmp.provider.storage.TceVolumeProvider;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.cmp.tceconvertor.RegionConvertor;
import com.bocloud.cmp.tceconvertor.ZoneConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesStatusRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesStatusResponse;
import com.tencentcloudapi.cvm.v20170312.models.DescribeRegionsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeRegionsResponse;
import com.tencentcloudapi.cvm.v20170312.models.DescribeZonesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeZonesResponse;
import com.tencentcloudapi.cvm.v20170312.models.InstanceStatus;
import com.tencentcloudapi.cvm.v20170312.models.RegionInfo;
import com.tencentcloudapi.cvm.v20170312.models.ZoneInfo;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TencentLocationProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceLocationProvider extends TceProvider implements LocationProvider {

	private static Logger logger = LoggerFactory.getLogger(TceLocationProvider.class);

	private Butler butler;
	private FlavorProvider flavorProvider = null;
	private ImageProvider imageProvider = null;
	private KeypairProvider keypairProvider = null;
	private SecurityGroupProvider secgProvider = null;
	private ServerProvider serverProvider = null;
	private SnapshotProvider snapshotProvider = null;
	private SubnetProvider subnetProvider = null;
	private VolumeProvider volumeProvider = null;
	private VpcProvider vpcProvider = null;
	private SecurityGroupRuleProvider securityGroupRuleProvider = null;

	public TceLocationProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	private BeanConvertor<RegionModel, RegionInfo> regionConvertor = new RegionConvertor();
	private BeanConvertor<ZoneModel, ZoneInfo> zoneConvertor = new ZoneConvertor();

	@Override
	public BocloudResult sync() {
		try {
			flavorProvider = new TceFlavorProvider(butler);
			imageProvider = new TceImageProvider(butler);
			keypairProvider = new TceKeypairProvider(butler);
			serverProvider = new TceServerProvider(butler);
			snapshotProvider = new TceSnapshotProvider(butler);
			volumeProvider = new TceVolumeProvider(butler);
			vpcProvider = new TceVpcProvider(butler);
			subnetProvider = new TceSubnetProvider(butler);
			secgProvider = new TceSecuGroupProvider(butler);
			securityGroupRuleProvider = new TceSecuGroupRuleProvider(butler);

			TceSyncModel syncModel = new TceSyncModel();
			List<RegionModel> regions = new ArrayList<>();
			List<FlavorModel> flavorModels = new ArrayList<>();
			List<ImageModel> imageModels = new ArrayList<>();
			List<VpcModel> vpcModels = new ArrayList<>();
			List<SecurityGroupModel> securityGroupModels = new ArrayList<>();
			List<VolumeModel> volumeModels = new ArrayList<>();
			List<ServerModel> serverModels = new ArrayList<>();
			List<SnapshotModel> snapshotModels = new ArrayList<>();
			List<SubnetModel> subnetModels = new ArrayList<>();
			List<KeypairModel> keypairModels = new ArrayList<>();

			logger.info("{} 地域 同步-->Start", this.butler.getName());
			RegionModel regionModel = new RegionModel();
			regionModel.setCode(this.getRegionId());
			if (!StringUtils.isEmpty(this.getRegionId())) {
				BocloudResult listZones = this.listZones(this.getRegionId());
				regionModel.setZones(JSONObject.parseArray(JSONObject.toJSONString(listZones.getData()),
						ZoneModel.class));
			}
			regions.add(regionModel);
			logger.info("{} 地域 同步-->End", this.butler.getName());
			Map<String, String> zoneMap = new HashMap<>();
			Optional.ofNullable(regionModel.getZones()).ifPresent(
					zones -> zones.stream().forEach(zone -> zoneMap.put(zone.getZoneNum(), zone.getZoneId())));
			logger.info("{} 规格 同步-->Start", this.butler.getName());
			BocloudResult flavorResult = flavorProvider.sync();
			if (flavorResult.isSuccess()) {
				flavorModels = JSONObject
						.parseArray(JSONObject.toJSONString(flavorResult.getData()), FlavorModel.class);
			}
			logger.info("{} 规格 同步-->End", this.butler.getName());
			logger.info("{} 镜像 同步-->Start", this.butler.getName());
			BocloudResult imageResult = imageProvider.list();
			if (imageResult.isSuccess()) {
				imageModels = JSONObject.parseArray(JSONObject.toJSONString(imageResult.getData()), ImageModel.class);
			}
			logger.info("{} 镜像 同步-->End", this.butler.getName());
			logger.info("{} 硬盘 同步-->Start", this.butler.getName());
			BocloudResult volumeResult = volumeProvider.list();
			if (volumeResult.isSuccess()) {
				volumeModels = JSONObject
						.parseArray(JSONObject.toJSONString(volumeResult.getData()), VolumeModel.class);
			}
			logger.info("{} 硬盘 同步-->End", this.butler.getName());
			logger.info("{} 快照 同步-->Start", this.butler.getName());
			BocloudResult snapshotResult = snapshotProvider.list();
			if (snapshotResult.isSuccess()) {
				snapshotModels = JSONObject.parseArray(JSONObject.toJSONString(snapshotResult.getData()),
						SnapshotModel.class);
			}
			logger.info("{} 快照 同步-->End", this.butler.getName());
			logger.info("{} 密钥 同步-->Start", this.butler.getName());
			BocloudResult keypairResult = keypairProvider.list();
			if (keypairResult.isSuccess()) {
				keypairModels = JSONObject.parseArray(JSONObject.toJSONString(keypairResult.getData()),
						KeypairModel.class);
			}
			logger.info("{} 密钥 同步-->End", this.butler.getName());
			logger.info("{} vpc 同步-->Start", this.butler.getName());
			BocloudResult vpcResult = vpcProvider.list();
			if (vpcResult.isSuccess()) {
				vpcModels = JSONObject.parseArray(JSONObject.toJSONString(vpcResult.getData()), VpcModel.class);
			}
			logger.info("{} vpc 同步-->End", this.butler.getName());
			logger.info("{} 子网 同步-->Start", this.butler.getName());
			BocloudResult subnetResult = subnetProvider.list();
			if (subnetResult.isSuccess()) {
				subnetModels = JSONObject
						.parseArray(JSONObject.toJSONString(subnetResult.getData()), SubnetModel.class);
				subnetModels.stream().forEach(model -> model.setZone(zoneMap.get(model.getZoneNum())));
			}
			logger.info("{} 子网 同步-->End", this.butler.getName());
			logger.info("{} 安全组 同步-->Start", this.butler.getName());
			BocloudResult securityResult = secgProvider.list();
			if (securityResult.isSuccess()) {
				securityGroupModels = JSONObject.parseArray(JSONObject.toJSONString(securityResult.getData()),
						SecurityGroupModel.class);
				securityGroupModels.stream().forEach(
						model -> {
							BocloudResult secRuleResult = securityGroupRuleProvider.list(model.getGroupUuid());
							if (secRuleResult.isSuccess()) {
								model.setModels(JSONObject.parseArray(JSONObject.toJSONString(secRuleResult.getData()),
										SecurityGroupRuleModel.class));
							}
						});
			}
			logger.info("{} 安全组 同步-->End", this.butler.getName());
			logger.info("{} 虚拟机 同步-->Start", this.butler.getName());
			BocloudResult serverResult = serverProvider.list();
			if (serverResult.isSuccess()) {
				serverModels = JSONObject
						.parseArray(JSONObject.toJSONString(serverResult.getData()), ServerModel.class);
				// for (ServerModel serverModel : serverModels) {
				// BocloudResult statusResult =
				// this.instancesStatus(serverModel.getInstanceId());
				// String serverStatus = (String) statusResult.getData();
				// serverModel.setStatus(serverStatus);
				// }
			}
			logger.info("{} 虚拟机 同步-->End", this.butler.getName());
			syncModel.setFlavorModels(flavorModels);
			syncModel.setRegionModels(regions);
			syncModel.setImageModels(imageModels);
			syncModel.setServerModels(serverModels);
			syncModel.setVolumeModels(volumeModels);
			syncModel.setSnapshotModels(snapshotModels);
			syncModel.setVpcModels(vpcModels);
			syncModel.setSubnetModels(subnetModels);
			syncModel.setSecurityGroupModels(securityGroupModels);
			syncModel.setKeypairModels(keypairModels);
			return new BocloudResult(true, syncModel, "success");
		} catch (Exception e) {
			logger.error("TCE同步失败： " + e);
			return new BocloudResult(false, e.getMessage(), "TCE同步失败");
		}
	}

	@Override
	public BocloudResult listRegions() {
		try {
			DescribeRegionsRequest req = DescribeRegionsRequest.fromJsonString("{}", DescribeRegionsRequest.class);
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeRegionsResponse resp = client.DescribeRegions(req);
			RegionInfo[] regionSet = resp.getRegionSet();
			return new BocloudResult(true, regionConvertor.convertList(Arrays.asList(regionSet)), "查询成功");
		} catch (Exception e) {
			logger.error("查询失败： " + e);
			return new BocloudResult(false, e.getMessage(), "查询失败");
		}
	}

	@Override
	public BocloudResult listZones(String region) {
		try {
			DescribeZonesRequest req = new DescribeZonesRequest();
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			client.setRegion(region);
			DescribeZonesResponse resp = client.DescribeZones(req);
			ZoneInfo[] zoneSet = resp.getZoneSet();
			return new BocloudResult(true, zoneConvertor.convertList(Arrays.asList(zoneSet)), "查询成功");
		} catch (Exception e) {
			logger.error("查询失败 ： " + e);
			return new BocloudResult(false, e.getMessage(), "查询失败");
		}
	}

	@Override
	public BocloudResult syncPortByNetworkUuid(String networkUuid) {
		return null;
	}

	@Override
	public BocloudResult getHostModel(HostModel arg0) {
		return null;
	}

	// 用于查询主机开关机状态
	private BocloudResult instancesStatus(String instancesId) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeInstancesStatusRequest req = new DescribeInstancesStatusRequest();
			req.setInstanceIds(new String[] { instancesId });
			DescribeInstancesStatusResponse resp = client.DescribeInstancesStatus(req);
			InstanceStatus[] instanceStatusSet = resp.getInstanceStatusSet();
			InstanceStatus instanceStatus = instanceStatusSet[0];
			return new BocloudResult(true, instanceStatus.getInstanceState(), "查询状态成功");
		} catch (Exception e) {
			logger.error("查询状态失败： " + e);
			return new BocloudResult(false, "查询状态失败", e.getMessage());
		}
	}

}
