package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.TceProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tceconvertor.ImageConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.CreateImageRequest;
import com.tencentcloudapi.cvm.v20170312.models.DeleteImagesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeImagesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeImagesResponse;
import com.tencentcloudapi.cvm.v20170312.models.Filter;
import com.tencentcloudapi.cvm.v20170312.models.Image;
import com.tencentcloudapi.cvm.v20170312.models.ModifyImageAttributeRequest;
import com.tencentcloudapi.cvm.v20170312.models.ModifyImageSharePermissionRequest;
import com.tencentcloudapi.cvm.v20170312.models.SyncImagesRequest;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceImageProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceImageProvider extends TceProvider implements ImageProvider {

	public TceImageProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(TceImageProvider.class);

	private BeanConvertor<ImageModel, Image> convertor = new ImageConvertor();

	@Override
	public BocloudResult list() {
		Integer offset = 0;
		BocloudResult result;
		DescribeImagesRequest imageRequest = new DescribeImagesRequest();
		List<Image> images = new ArrayList<>();
		try {
			imageRequest.setLimit(LIMIT);
			DescribeImagesResponse response = ((CvmClient) this.getClient(TceEnum.CVM)).DescribeImages(imageRequest);
			images.addAll(Arrays.asList(response.getImageSet()));
			// 循环分页查询
			while (images.size() >= LIMIT + offset) {
				imageRequest.setOffset(offset += LIMIT);
				response = ((CvmClient) this.getClient(TceEnum.CVM)).DescribeImages(imageRequest);
				images.addAll(Arrays.asList(response.getImageSet()));
			}
			result = new BocloudResult(true, convertor.convertList(images), "success");
		} catch (Exception e) {
			logger.error("list images error:", e.getMessage());
			result = new BocloudResult(false, e.getMessage());
		}
		return result;
	}

	@Override
	public BocloudResult detail(String imageId) {
		try {
			DescribeImagesRequest req = new DescribeImagesRequest();
			req.setImageIds(new String[] { imageId });
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			DescribeImagesResponse resp = client.DescribeImages(req);
			ImageModel imageModel = convertor.convertModel(resp.getImageSet()[0]);
			imageModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, imageModel, "detail image success!");
		} catch (Exception e) {
			logger.error("detail image error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String imageId) {
		try {
			DeleteImagesRequest req = new DeleteImagesRequest();
			req.setImageIds(new String[] { imageId });
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			client.DeleteImages(req);
			return new BocloudResult(true, "remove image success!");
		} catch (Exception e) {
			String message = e.getMessage();
			if (null != message && message.contains("")) {
				return new BocloudResult(true, "删除成功！");
			}
			logger.error("remove image error :", message);
			return new BocloudResult(false, message);
		}
	}

	@Override
	public BocloudResult modify(ImageModel image) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			ModifyImageAttributeRequest request = new ModifyImageAttributeRequest();
			request.setImageId(image.getImageUuid());
			if (!StringUtils.isEmpty(image.getName())) {
				request.setImageName(image.getName());
			}
			if (!StringUtils.isEmpty(image.getRemark())) {
				request.setImageDescription(image.getRemark());
			}
			client.ModifyImageAttribute(request);
			return new BocloudResult(true, this.detail(image.getImageUuid()).getData(), "update image success!");
		} catch (Exception e) {
			logger.error("update image error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	public Image getImages(String imageName, String region) {
		Image image = null;
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			if (!StringUtils.isEmpty(region)) {
				client.setRegion(region);
			}
			// 加入筛选条件
			DescribeImagesRequest req = new DescribeImagesRequest();
			Filter filter = new Filter();
			filter.setName("image-type");
			filter.setValues(new String[] { "PRIVATE_IMAGE" });
			Filter[] filters = new Filter[] { filter };
			req.setFilters(filters);
			Long startTime = System.currentTimeMillis();
			Long endTime = System.currentTimeMillis();
			while (endTime - startTime < 5 * 60 * 1000) {
				// 创建时间不确定，增加休眠减少轮询次数
				Thread.sleep(6000);
				DescribeImagesResponse response = client.DescribeImages(req);
				Image[] images = response.getImageSet();
				if (null == images || images.length <= 0) {
					logger.error("data does not exist :", imageName);
					break;
				}
				for (Image img : images) {
					if (imageName.equalsIgnoreCase(img.getImageName())
							&& "NORMAL".equalsIgnoreCase(img.getImageState())) {
						image = img;
						break;
					}
				}
				if (!ObjectUtils.isEmpty(image)) {
					break;
				}
				endTime = System.currentTimeMillis();
			}
		} catch (Exception e) {
			logger.error("image get status error： " + e.getMessage());
		}
		return image;
	}

	@Override
	public BocloudResult create(ImageModel imageModel) {
		try {
			CvmClient client = (CvmClient) this.getClient(TceEnum.CVM);
			CreateImageRequest req = new CreateImageRequest();
			req.setInstanceId(imageModel.getInstanceId());
			req.setImageName(imageModel.getName());
			req.setImageDescription(imageModel.getRemark());
			client.CreateImage(req);
			Image image = getImages(imageModel.getName(), null);
			if (!ObjectUtils.isEmpty(image)) {
				ImageModel model = convertor.convertModel(image);
				model.setRegionId(this.getRegionId());
				return new BocloudResult(true, model, "create image success!");
			} else {
				return new BocloudResult(false, "create image timeout! ");
			}
		} catch (Exception e) {
			logger.error("create image error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult shareImage(ImageModel imageModel) {
		try {
			ModifyImageSharePermissionRequest request = new ModifyImageSharePermissionRequest();
			request.setImageId(imageModel.getImageUuid());
			request.setAccountIds(imageModel.getAccountIds());
			request.setPermission(imageModel.getPermission());
			((CvmClient) this.getClient(TceEnum.CVM)).ModifyImageSharePermission(request);
			return new BocloudResult(true, "share image success!");
		} catch (Exception e) {
			logger.error("share image error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult syncImages(ImageModel imageModel) {
		try {
			SyncImagesRequest request = new SyncImagesRequest();
			request.setDestinationRegions(new String[] { imageModel.getRegionId() });
			request.setImageIds(new String[] { imageModel.getImageUuid() });
			((CvmClient) this.getClient(TceEnum.CVM)).SyncImages(request);
			Image image = this.getImages(imageModel.getName(), imageModel.getRegionId());
			if (!ObjectUtils.isEmpty(image)) {
				return new BocloudResult(true, convertor.convertModel(image), "cross-domain image success!");
			} else {
				return new BocloudResult(false, "cross-domain image timeout! ");
			}
		} catch (Exception e) {
			logger.error("cross-domain image error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

}
