package com.bocloud.cmp.provider.compute;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;

import org.apache.commons.lang3.ObjectUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.TceProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tceconvertor.SnapshotConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cbs.v20170312.CbsClient;
import com.tencentcloudapi.cbs.v20170312.models.ApplySnapshotRequest;
import com.tencentcloudapi.cbs.v20170312.models.DeleteSnapshotsRequest;
import com.tencentcloudapi.cbs.v20170312.models.DescribeSnapshotsRequest;
import com.tencentcloudapi.cbs.v20170312.models.DescribeSnapshotsResponse;
import com.tencentcloudapi.cbs.v20170312.models.ModifySnapshotAttributeRequest;
import com.tencentcloudapi.cbs.v20170312.models.Snapshot;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: TceSnapshotProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.tce")
public class TceSnapshotProvider extends TceProvider implements SnapshotProvider {

    private static Logger logger = LoggerFactory.getLogger(TceSnapshotProvider.class);

    private BeanConvertor<SnapshotModel, Snapshot> convertor = new SnapshotConvertor();

    public TceSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
            DescribeSnapshotsResponse response = client.DescribeSnapshots(new DescribeSnapshotsRequest());
            List<SnapshotModel> models = convertor.convertList(Arrays.asList(response.getSnapshotSet()));
            Optional.ofNullable(models).ifPresent(snapshotModels -> snapshotModels.stream().forEach(snapshotModel -> snapshotModel.setRegionId(this.getRegionId())));
            return new BocloudResult(true, models, "list snapshot success!");
        } catch (Exception e) {
            logger.error("list snapshot error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
            DeleteSnapshotsRequest req = new DeleteSnapshotsRequest();
            req.setSnapshotIds(new String[]{model.getSnapshotUuid()});
            client.DeleteSnapshots(req);
            return new BocloudResult(true, "remove snapshot success!");
        } catch (Exception e) {
            logger.error("remove snapshot error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
            DescribeSnapshotsRequest req = new DescribeSnapshotsRequest();
            req.setSnapshotIds(new String[]{model.getSnapshotUuid()});
            DescribeSnapshotsResponse resp = client.DescribeSnapshots(req);
            Snapshot[] snapshotSet = resp.getSnapshotSet();
            SnapshotModel convertModel = null;
            if (null != snapshotSet && snapshotSet.length > 0) {
                convertModel = convertor.convertModel(snapshotSet[0]);
                convertModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertModel, "detail snapshot success!");
        } catch (Exception e) {
            logger.error("detail snapshot error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        try {
            CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
            ModifySnapshotAttributeRequest request = new ModifySnapshotAttributeRequest();
            request.setSnapshotId(model.getSnapshotUuid());
            request.setSnapshotName(model.getName());
            client.ModifySnapshotAttribute(request);
            return new BocloudResult(true, "modify snapshot success!");
        } catch (Exception e) {
            logger.error("modify snapshot error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        try {
            CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
            ApplySnapshotRequest request = new ApplySnapshotRequest();
            request.setDiskId(model.getVolumeUuid());
            request.setSnapshotId(model.getSnapshotUuid());
            client.ApplySnapshot(request);
            return new BocloudResult(true, "revert snapshot success!");
        } catch (Exception e) {
            logger.error("revert snapshot error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        BocloudResult result = new BocloudResult(false, "data does not exist");
        SnapshotModel snapshotModel = new SnapshotModel();
        snapshotModel.setSnapshotUuid(snapshotId);
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 5 * 60 * 1000) {
                //创建时间不确定，增加休眠减少轮询次数
                Thread.sleep(6000);
                BocloudResult detail = this.detail(snapshotModel);
                SnapshotModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), SnapshotModel.class);
                if (ObjectUtils.isEmpty(model)) {
                    logger.error("data does not exist :", snapshotId);
                    break;
                }
                if ("NORMAL".equalsIgnoreCase(model.getStatus()) || "AVAILABLE".equalsIgnoreCase(model.getStatus())) {
                    result = this.detail(snapshotModel);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error("create snapshot get status error： " + e.getMessage());
            result = new BocloudResult(false, "create snapshot get status error： " + e.getMessage());
        }
        return result;
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult list(Long arg0) {
        return null;
    }

}
