package com.bocloud.cmp.provider.storage;

import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.TceProvider;
import com.bocloud.cmp.provider.compute.TceSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tceconvertor.VolumeConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cbs.v20170312.CbsClient;
import com.tencentcloudapi.cbs.v20170312.models.AttachDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.AttachDisksResponse;
import com.tencentcloudapi.cbs.v20170312.models.CreateDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.CreateDisksResponse;
import com.tencentcloudapi.cbs.v20170312.models.CreateSnapshotRequest;
import com.tencentcloudapi.cbs.v20170312.models.CreateSnapshotResponse;
import com.tencentcloudapi.cbs.v20170312.models.DescribeDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.DescribeDisksResponse;
import com.tencentcloudapi.cbs.v20170312.models.DetachDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.Disk;
import com.tencentcloudapi.cbs.v20170312.models.DiskChargePrepaid;
import com.tencentcloudapi.cbs.v20170312.models.Filter;
import com.tencentcloudapi.cbs.v20170312.models.ModifyDiskAttributesRequest;
import com.tencentcloudapi.cbs.v20170312.models.Placement;
import com.tencentcloudapi.cbs.v20170312.models.ResizeDiskRequest;
import com.tencentcloudapi.cbs.v20170312.models.TerminateDisksRequest;

/**
 * 卷操作类
 *
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tce")
public class TceVolumeProvider extends TceProvider implements VolumeProvider {

	private static Logger logger = LoggerFactory.getLogger(TceVolumeProvider.class);

	private BeanConvertor<VolumeModel, Disk> convertor = new VolumeConvertor();

	private Butler butler;

	public TceVolumeProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		try {
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			DescribeDisksResponse response = client.DescribeDisks(new DescribeDisksRequest());
			return new BocloudResult(true, convertor.convertList(Arrays.asList(response.getDiskSet())),
					"list disk success!");
		} catch (Exception e) {
			logger.error("list disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	public BocloudResult list(String instanceId) {
		try {
			DescribeDisksRequest req = new DescribeDisksRequest();
			Filter f = new Filter();
			f.setName("instance-id");
			f.setValues(new String[] { instanceId });
			Filter[] filters = new Filter[] { f };
			req.setFilters(filters);
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			DescribeDisksResponse resp = client.DescribeDisks(req);
			Disk[] diskSet = resp.getDiskSet();
			return new BocloudResult(true, convertor.convertList(Arrays.asList(diskSet)), "list disk success!");
		} catch (Exception e) {
			logger.error("list disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(VolumeModel volume) {
		try {
			CreateDisksRequest req = new CreateDisksRequest();
			// CLOUD_BASIC：表示普通云硬盘
			// CLOUD_PREMIUM：表示高性能云硬盘
			// CLOUD_SSD：表示SSD云硬盘。
			req.setDiskType(volume.getType());
			// PREPAID：预付费，即包年包月
			// POSTPAID_BY_HOUR：按小时后付费
			String diskChargeType = volume.getDiskChargeType();
			if (null == diskChargeType || "".equals(diskChargeType)) {
				diskChargeType = "POSTPAID_BY_HOUR";
			}
			req.setDiskChargeType(diskChargeType); // 按小时计费
			volume.setDiskChargeType(diskChargeType);
			if ("PREPAID".equalsIgnoreCase(diskChargeType)) {
				DiskChargePrepaid diskChargePrepaid = new DiskChargePrepaid();
				Integer period = volume.getPeriod();
				if (null == period || period <= 0) {
					period = 1;
				}
				diskChargePrepaid.setPeriod(Integer.valueOf(period));
				req.setDiskChargePrepaid(diskChargePrepaid);
			}
			Placement placement = new Placement();
			String proId = this.getProjectId();
			if (null != proId && !"".equals(proId)) {
				placement.setProjectId(Integer.valueOf(this.getProjectId()));
			}
			placement.setZone(volume.getZone());
			req.setPlacement(placement);
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			if (null != volume.getSourceSnapshotId() && !"".equals(volume.getSourceSnapshotId())) {
				req.setSnapshotId(volume.getSourceSnapshotId());
			}
			req.setDiskSize(Integer.valueOf(volume.getSize()));
			req.setDiskName(volume.getName());
			CreateDisksResponse createDisks = client.CreateDisks(req);
			volume.setVolumeUuid(createDisks.getDiskIdSet()[0]);
			volume.setRegionId(this.getRegionId());
			return new BocloudResult(true, volume, "create disk success!");
		} catch (Exception e) {
			logger.error("create disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(VolumeModel volume) {
		try {
			ModifyDiskAttributesRequest req = new ModifyDiskAttributesRequest();
			req.setDiskName(volume.getName());
			req.setDeleteWithInstance(volume.getDeleteWithInstance());
			req.setDiskIds(new String[] { volume.getVolumeUuid() });
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			client.ModifyDiskAttributes(req);
			return new BocloudResult(true, "modify disk success!");
		} catch (Exception e) {
			logger.error("modify disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String volumeId) {
		try {
			TerminateDisksRequest req = new TerminateDisksRequest();
			req.setDiskIds(new String[] { volumeId });
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			client.TerminateDisks(req);
			return new BocloudResult(true, "remove disk success!");
		} catch (Exception e) {
			logger.error("remove disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String volumeId) {
		try {
			DescribeDisksRequest req = new DescribeDisksRequest();
			req.setDiskIds(new String[] { volumeId });
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			DescribeDisksResponse resp = client.DescribeDisks(req);
			Disk[] diskSet = resp.getDiskSet();
			VolumeModel volumeModel = convertor.convertModel(diskSet[0]);
			volumeModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, volumeModel, "detail disk success!");
		} catch (Exception e) {
			logger.error("detail disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		try {
			CreateSnapshotRequest req = new CreateSnapshotRequest();
			req.setDiskId(snapshotModel.getVolumeUuid());
			req.setSnapshotName(snapshotModel.getName());
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			CreateSnapshotResponse resp = client.CreateSnapshot(req);
			return new BocloudResult(true, this.snapshotDetail(resp.getSnapshotId()).getData(),
					"create snapshot success!");
		} catch (Exception e) {
			logger.error("create snapshot error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	private BocloudResult snapshotDetail(String snapshotId) {
		TceSnapshotProvider provider = new TceSnapshotProvider(butler);
		SnapshotModel model = new SnapshotModel();
		model.setSnapshotUuid(snapshotId);
		BocloudResult detail = provider.detail(model);
		return detail;
	}

	@Override
	public BocloudResult attach(String serverId, String volumeId) {
		try {
			AttachDisksRequest req = new AttachDisksRequest();
			req.setDiskIds(new String[] { volumeId });
			req.setInstanceId(serverId);
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			AttachDisksResponse response = client.AttachDisks(req);
			return new BocloudResult(true, response.getRequestId(), "attach disk success!");
		} catch (Exception e) {
			logger.error("attach disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
		return null;
	}

	@Override
	public BocloudResult detach(String serverId, String volumeId) {
		try {
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			DetachDisksRequest req = new DetachDisksRequest();
			req.setDiskIds(new String[] { volumeId });
			client.DetachDisks(req);
			return new BocloudResult(true, "detach disk success!");
		} catch (Exception e) {
			logger.error("detach disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult resetStatus(String volumeId, String status) {
		return null;
	}

	@Override
	public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
		return null;
	}

	@Override
	public BocloudResult extend(String volumeId, Integer newSize) {
		try {
			ResizeDiskRequest req = new ResizeDiskRequest();
			req.setDiskId(volumeId);
			req.setDiskSize(Integer.valueOf(newSize));
			CbsClient client = (CbsClient) this.getClient(TceEnum.CBS);
			client.ResizeDisk(req);
			return new BocloudResult(true, "extend disk success!");
		} catch (Exception e) {
			logger.error("extend disk error :", e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult list(String hostId, String storagePoolName) {
		return null;
	}

	@Override
	public BocloudResult remove(H3cVolumeModel arg0) {
		return null;
	}

}
